package com.ruoyi.system.service.impl;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.system.entity.CommonEntity;
import com.ruoyi.system.entity.TableSql;
import com.ruoyi.system.mapper.CommonMapper;
import com.ruoyi.system.service.ICommonService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

import static com.ruoyi.common.utils.StringUtils.convertToCamelCase;
import static com.ruoyi.common.utils.StringUtils.toUnderScoreCase;


@Service
public class CommonServiceImpl implements ICommonService {

    @Resource
    private CommonMapper commonMapper;


    @Override
    public List<CommonEntity> selectList(Map<String, Object> map) {
        String tableName = (String) map.get("tableName");
        Map<String, Object> conditions = JSONObject.parseObject(JSON.toJSONString(map.get("condition")));
        return commonMapper.selectList(tableName, conditions);
    }

    @Override
    public int batchInsert(Map<String, Object> map) {
        String tableName = (String) map.get("tableName");
        List<Object> list = (List<Object>) map.get("field");
        List<Map<String, Object>> mapList = new ArrayList<>();
        for (Object obj : list) {
            Map<String, Object> map1 = JSONObject.parseObject(JSON.toJSONString(obj));
            mapList.add(map1);
        }
        Set<String> fieldNames = new LinkedHashSet<>();
        mapList.stream().forEach(map1 -> {
            map1.forEach((key, value) -> {
                fieldNames.add(key);
            });
        });
        return commonMapper.batchInsert(fieldNames, tableName, mapList);
    }

    @Override
    public int edit(Map<String, Object> map) {
        String tableName = (String) map.get("tableName");
        Map<String, Object> conditions = JSONObject.parseObject(JSON.toJSONString(map.get("condition")));
        Map<String, Object> fields = JSONObject.parseObject(JSON.toJSONString(map.get("field")));
        return commonMapper.edit(fields, tableName, conditions);
    }

    @Override
    public int batchDelete(Map<String, Object> map) {
        String tableName = (String) map.get("tableName");
        Map<String, Object> conditions = JSONObject.parseObject(JSON.toJSONString(map.get("condition")));
        return commonMapper.batchDelete(tableName, conditions);
    }

    @Override
    public List<CommonEntity> queryTableList(CommonEntity commonEntity, TableSql tableSql) {
        // 根据sqlkey查询得到当前表单对应的sql
        Map<String, Object> conditions = JSONObject.parseObject(JSON.toJSONString(commonEntity.getCondition()));
        // sql当中拼接好条件格式 后台使用AND分割 拼接条件
        //  and 。。。。 and 。。。。 and。。。。
        StringBuilder endSQL = new StringBuilder();
        endSQL.append("" + tableSql.getTableSql() + " where 1=1 ");
        String[] whereTerm = tableSql.getTableCondition().split("AND");
        List<String> test = Arrays.asList(whereTerm);
        // 拼接条件
        conditions.forEach((key, val) -> {
            // 把前端回传的查询参数值名称驼峰转下划线命名，进行筛选是否存在匹配项
            String sqlWhere = test.stream().filter(item -> item.contains(toUnderScoreCase(key))).findAny().get();
            if (sqlWhere.indexOf("#{timeframe}") != -1) {  // 时间范围查询拼接
                endSQL.append(" AND " + sqlWhere.replace("#{timeframe}", "'" + val.toString().split(",")[0] + "'" + " and " + "'" + val.toString().split(",")[1] + "'"));
            } else if (sqlWhere.indexOf("#{val}") != -1) { // 寻常条件拼接
                endSQL.append(" AND " + sqlWhere.replace("#{val}", val.toString()));
            }
        });
        //for (String w : whereTerm) {
        //extractSubstring(w, tableSql.getTableAlias()); // 得到当前筛选条件的字段名称
        //w.replace("#{val}", extractSubstring(w, tableSql.getTableAlias()));
        //}
        //conditions.forEach((key, val) -> {
        //endSQL.append(" and " + tableSql.getTableAlias() + " " + key + " = " + val);
        //});
        return commonMapper.queryTableList(endSQL.toString());
    }


    public static String extractSubstring(String input, String identifier) {
        int startIndex = input.indexOf(identifier);
        if (startIndex == -1) {
            return null; // 如果找不到指定的标识符，则返回空
        }

        int endIndex = input.indexOf(" ", startIndex);
        if (endIndex == -1) {
            endIndex = input.length(); // 如果找不到空格，则截取到字符串末尾
        }
        String sqlColumn = input.substring(startIndex + identifier.length() + 1, endIndex);
        return convertToCamelCase(sqlColumn);

    }
}
